// Licensed to Apache Software Foundation (ASF) under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Apache Software Foundation (ASF) licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package meter_test

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"github.com/apache/skywalking-banyandb/pkg/meter"
)

// TestHierarchicalScope tests the hierarchical scope.
func TestHierarchicalScope(t *testing.T) {
	sep := "."
	scope := meter.NewHierarchicalScope("test", sep)
	scope = scope.ConstLabels(meter.LabelPairs{
		"env":     "dev",
		"version": "1.0",
	})
	scope = scope.SubScope("child")
	scope = scope.ConstLabels(meter.LabelPairs{
		"version": "2.0",
	})
	scope = scope.SubScope("grandchild")
	scope = scope.ConstLabels(meter.LabelPairs{
		"version": "3.0",
	})
	hs, ok := scope.(*meter.HierarchicalScope)
	require.True(t, ok)
	assert.Equal(t, "test.child.grandchild", hs.GetNamespace())
	assert.Equal(t, "dev", hs.GetLabels()["env"])
	assert.Equal(t, "3.0", hs.GetLabels()["version"])
}
